package net.wanji.opt.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossDataRealtimeMapper;
import net.wanji.databus.dao.mapper.CrossDirDataRealtimeMapper;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossDirDataRealtimePO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.opt.common.enums.CrossOptStrategyEnum;
import net.wanji.opt.dao.mapper.StrategyCrossResultMapper;
import net.wanji.opt.service.CrossIndexService;
import net.wanji.opt.synthesis.pojo.StrategyCrossResultEntity;
import net.wanji.opt.vo.CrossOptResult;
import org.drools.core.util.DateUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/25 19:12
 */
@Service
@Slf4j
public class CrossIndexServiceImpl implements CrossIndexService {

    @Resource
    private CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;
    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;
    @Resource
    private StrategyCrossResultMapper strategyCrossResultMapper;

    @Override
    public Map<Integer, CrossDirDataRealtimePO> crossDirIndex(CrossIdBO crossIdBO) {
        List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = crossDirDataRealtimeMapper.selectBycrossId(crossIdBO.getCrossId());
        if (!CollectionUtils.isEmpty(crossDirDataRealtimePOS)) {
            Map<Integer, CrossDirDataRealtimePO> dirDataMap = new HashMap<>(crossDirDataRealtimePOS.size());
            for (CrossDirDataRealtimePO crossDirDataRealtimePO : crossDirDataRealtimePOS) {
                dirDataMap.put(crossDirDataRealtimePO.getDirType(), crossDirDataRealtimePO);
            }
            return dirDataMap;
        }
        return null;
    }

    @Override
    public CrossDataRealtimePO crossIndex(CrossIdBO crossIdBO) {
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossIdBO.getCrossId());
        return crossDataRealtimePO;
    }


    @Override
    public List<TBaseCrossInfo> crossInfoList() {
        LambdaQueryWrapper<TBaseCrossInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TBaseCrossInfo::getIsSignal, 1);
        List<TBaseCrossInfo> baseCrossInfoPOS = baseCrossInfoMapper.selectList(queryWrapper);
        return baseCrossInfoPOS;
    }

    @Override
    public List<CrossOptResult> crossOptResultList(CrossIdBO crossIdBO) throws Exception{
        LambdaQueryWrapper<StrategyCrossResultEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, crossIdBO.getCrossId());
        LocalDate currentDate = LocalDate.now();
        LocalTime startTime = LocalTime.MIDNIGHT;
        LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
        queryWrapper.eq(StrategyCrossResultEntity::getCrossId, crossIdBO.getCrossId());
        queryWrapper.ge(StrategyCrossResultEntity::getIssueTime, startOfDay);
        List<StrategyCrossResultEntity> list = strategyCrossResultMapper.selectList(queryWrapper);
        List<CrossOptResult> crossOptResults = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list)) {
            for (StrategyCrossResultEntity entity : list) {
                CrossOptResult crossOptResult = new CrossOptResult();
                Date date =DateUtil.parse(entity.getIssueTime(), "yyyy-MM-dd HH:mm:ss");
                crossOptResult.setTimeStamp(date);
                String timingPlan = entity.getTimingPlan();
                Integer countDown = entity.getCountDown();
                if (entity.getCurrentAlgo() == 2) {
                    countDown = 5;
                }
                crossOptResult.setCountDown(countDown);
                Integer currentAlgo = entity.getCurrentAlgo();
                crossOptResult.setStrategy(currentAlgo);
                crossOptResult.setStrategyName(CrossOptStrategyEnum.getDesc(currentAlgo));
                crossOptResults.add(crossOptResult);
            }
        }
        return crossOptResults;
    }
}
