package net.wanji.opt.service.induce.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.entity.DeviceInduces;
import net.wanji.opt.entity.GreenwaveInduces;
import net.wanji.opt.entity.InduceHist;
import net.wanji.opt.entity.InduceTemplate;
import net.wanji.opt.dto.induce.MessageParam;
import net.wanji.opt.service.DeviceInducesService;
import net.wanji.opt.service.induce.*;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Objects;
import java.util.UUID;

@Service
@Slf4j
public class InduceSendServiceImpl implements InduceSendService {

    @Autowired
    private InduceTemplateService induceTemplateService;

    @Autowired
    RabbitTemplate rabbitTemplate;

    @Autowired
    private InduceHistService induceHistService;

    @Autowired
    private GreenwaveInducesService greenwaveInducesService;

    @Autowired
    private DeviceInducesService deviceInducesService;

    @Autowired
    private Configuration freemarkerConfig;

    @Value("${ftp.url}")
    private String FTP_IP;

    @Value("${ftp.username}")
    private String FTP_USERNAME;

    @Value("${ftp.password}")
    private String FTP_PASSWORD;

    @Value("${ftp.port}")
    private Integer FTP_PORT;

    @Value("${ftp.path}")
    private String PATH;

    @Value("${spring.rabbitmq.queue-name}")
    private String queueName;
    /**
     * 是否下发屏幕
     */
    @Value("${spring.rabbitmq.sending}")
    private Boolean sending=false;

    private FTPClient ftpClient;

    @Override
    public void send(MessageParam param) throws IOException, TemplateException {
        //生成诱导屏推送sourceId
        String messageId = UUID.randomUUID().toString().toLowerCase();
        InduceHist induceHist=new InduceHist();
        param.setSourceId(messageId);
        induceHist.setSourceId(messageId);
        GreenwaveInduces greenwaveinduces=greenwaveInducesService.getById(param.getInduceId());
        // 读取图片模板参数
        InduceTemplate induceTemplate = induceTemplateService.getById(param.getTemplateId());
        if(param.getFlg()==1) {
            LambdaQueryWrapper<DeviceInduces> deviceQueryWrapper = new LambdaQueryWrapper<>();
            deviceQueryWrapper.eq(DeviceInduces::getEquipCode, induceTemplate.getEquipCode());
            DeviceInduces deviceInduces = deviceInducesService.getOne(deviceQueryWrapper);
            if(Objects.nonNull(deviceInduces)) {
                param.setResolutionWidth(deviceInduces.getResolutionWidth().intValue());
                param.setResolutionHeight(deviceInduces.getResolutionHeight().intValue());
            }
        }
        // 获取图片字节数组
        byte[] imageBytes = induceTemplate.getFileContent();
        if (imageBytes != null || imageBytes.length > 0) {
            // 将字节数组转换为字节对象
            BufferedImage image = ImageIO.read(new ByteArrayInputStream(imageBytes));
            // 修改图像中文字
            //addTextToImage(image, param.getContents(), induceTemplate.getTextX(), induceTemplate.getTextY(), induceTemplate.getTextSize(), induceTemplate.getTextSign());
            // 定义文字区域四个角的坐标
            String[] topLeft = induceTemplate.getTopLeft().split(",");
            String[] bottomRight = induceTemplate.getBottomRight().split(",");
            if (topLeft.length != 2 || bottomRight.length != 2) {
                return;
            }
            int topLeftX = Integer.parseInt(topLeft[0]);
            int topLeftY = Integer.parseInt(topLeft[1]);
            int bottomRightX = Integer.parseInt(bottomRight[0]);
            int bottomRightY = Integer.parseInt(bottomRight[1]);
            // 图片生成操作
            boolean upDown=param.getContents()[0].contains("▲") || param.getContents()[0].contains("▼");
            createImageWithText(induceTemplate, image, param.getContents(), induceTemplate.getTextSign(), upDown,topLeftX, topLeftY, bottomRightX, bottomRightY);
            // 测试阶段保存图片到本地
//            ImageIO.write(image, "bmp", new File("D:\\tmp\\" + induceTemplate.getFileName()));
            // 上传文件至ftp
            if (param.getFlg() == 1) {
                String dirName=LocalDate.now().format(DateTimeFormatter.ofPattern("yyyyMMdd"));
                String fileName=param.getEquipCode()+"_"+UUID.randomUUID().toString()+".bmp";
                String filePath = uploadFile(image,dirName, fileName);
                param.setFtpPath(filePath);
                induceHist.setFilePath(PATH + dirName+"/"+fileName);
                if(Objects.nonNull(param.getContents()) && param.getContents().length>0) {
                    induceHist.setInduceSpeed(param.getContents()[0]);
                }
                if(Objects.nonNull(greenwaveinduces)) {
                    greenwaveinduces.setSourceId(PATH + dirName + "/" + fileName);
                }
            }
            log.info("上传文件至ftp-filePath:{}", param.getFtpPath());
//            //如果已发布，则调用下屏服务
//            if(greenwaveinduces.getStatus()==1){
//                param.setFlg(2);
//                param.setSourceId(greenwaveinduces.getSourceId());
//                // 推送mq消息
//                String message = convertMessage(param.getFlg()==1?"induce_up.xml":"induce_down.xml",param);
//                // 去掉所有转义字符（\n、\t、\b、等）
////                message = message.replaceAll("[\\n\\t\\r]", ""); // 可以根据需要添加更多字符
//                // 可以去掉反斜杠
////                message = message.replace("\\", "");
//                // 创建MessageProperties并设置内容类型
//                MessageProperties messageProperties = new MessageProperties();
//                messageProperties.setContentType("text/plain"); // 设置内容类型为text/plain
//                // 创建Message并设置内容
//                Message messageSend = new Message(message.getBytes(), messageProperties);
//                rabbitTemplate.convertAndSend("",queueName, messageSend);
//            }
            // 推送mq消息
            String message = convertMessage(param.getFlg()==1?"induce_up.xml":"induce_down.xml",param);
            // 去掉所有转义字符（\n、\t、\b、等）
//            message = message.replaceAll("[\\n\\t\\r]", ""); // 可以根据需要添加更多字符
            // 可以去掉反斜杠
//            message = message.replace("\\", "");
            // 创建MessageProperties并设置内容类型
//            MessageProperties messageProperties = new MessageProperties();
//            messageProperties.setContentType("text/plain"); // 设置内容类型为text/plain
            // 创建Message并设置内容
//            Message messageSend = new Message(message.getBytes(), messageProperties);
            if(sending)
            {
                rabbitTemplate.convertAndSend("", queueName, message);
            }
            log.info("mqtt发送信息-topic:{},queueName:{},message:{}", queueName, message);
            //1、增加下发记录log 2、计数 3、标记已发布状体
            induceHist.setEquipCode(induceTemplate.getEquipCode());
            induceHist.setGreenId(param.getGreenId());
            induceHistService.save(induceHist);
            //保存诱导状态信息
            if(Objects.nonNull(greenwaveinduces)) {
                greenwaveinduces.setInducesCount(Objects.isNull(greenwaveinduces.getInducesCount()) ? 0 : greenwaveinduces.getInducesCount() + 1);
                greenwaveinduces.setStatus(1);
                greenwaveInducesService.saveOrUpdate(greenwaveinduces);
            }
        }
    }

    @Override
    public String uploadFile(BufferedImage image,String directoryName, String fileName) {
        ByteArrayInputStream inputStream = null;
        try {
            ftpClient = new FTPClient();
            //优先设置编码格式，避免中文乱码
            ftpClient.setAutodetectUTF8(true);
            ftpClient.setControlEncoding(StandardCharsets.UTF_8.name());
            ftpClient.setCharset(StandardCharsets.UTF_8);

            //连接ftp和登录ftp服务器
            ftpClient.connect(FTP_IP, FTP_PORT);
            ftpClient.login(FTP_USERNAME, FTP_PASSWORD);
            log.info("ftp服务器连接成功");

            //进入本地被动模式
            ftpClient.enterLocalPassiveMode();
            //将设置文件传输模式是二进制传输，可以保证传输的内容不会被改变
            ftpClient.setFileType(FTP.BINARY_FILE_TYPE);

            makeDir(ftpClient, PATH + directoryName);
            log.info("ftp切换工作目录成功, 当前工作目录是:{}", ftpClient.printWorkingDirectory());
            // 将BufferedImage转换为ByteArrayOutputStream
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ImageIO.write(image, "jpg", baos);
            baos.flush();
            inputStream = new ByteArrayInputStream(baos.toByteArray());
            //上传文件到ftp服务器上
            boolean hasUploaded = ftpClient.storeFile(fileName, inputStream);
            if (hasUploaded) {
                log.info("{}文件上传成功", fileName);
                return "ftp://" + FTP_USERNAME + ":" + FTP_PASSWORD +"@"+ FTP_IP + PATH + directoryName + "/" + fileName;
            }
            log.info("ftp服务器上传文件{}失败", fileName);
            return null;
        } catch (Exception e) {
            log.error(e.getMessage());
            return null;
        } finally {
            if (ftpClient.isConnected()) {
                try {
                    ftpClient.disconnect();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

            if (null != inputStream) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @Override
    public byte[] downloadImage(String remoteFile) throws IOException {
        // 远程文件路径
        String filePath = remoteFile;
        // 创建 FTP 客户端
        FTPClient ftpClient = new FTPClient();
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            // FTP 服务器地址
            ftpClient.connect(FTP_IP, FTP_PORT);
            // 用户名和密码
            ftpClient.login(FTP_USERNAME, FTP_PASSWORD);
            ftpClient.enterLocalPassiveMode();
            ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
            // 获取文件字节数组
            InputStream inputStream = ftpClient.retrieveFileStream(filePath);
            if (inputStream != null) {
                byte[] buffer = new byte[4096];
                int bytesRead;

                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
                // 完成文件传输
                ftpClient.completePendingCommand();
                // 返回字节数组
                return outputStream.toByteArray();
            }
        } finally {
            outputStream.close();
            ftpClient.logout();
            ftpClient.disconnect();
        }
        return null; // 如果下载失败返回 null
    }

    private void makeDir(FTPClient ftp, String path) throws IOException {
        String[] paths = path.trim().split("/");
        for (String directory : paths) {
            if (directory.length() > 0 && !ftp.changeWorkingDirectory(directory)) {
                ftp.makeDirectory(directory);
                ftp.changeWorkingDirectory(directory);
            }
        }
    }


    /**
     * 根据绘制的区域，动态调整字体大小，以适应文本区域
     *
     * @param g2d    graphics2d对象
     * @param sign   绘制方向标识
     * @param text   绘制内容
     * @param width  绘制区域宽度
     * @param height 绘制区域高度
     */
    private static FontMetrics autoFont(InduceTemplate pictureFile, Graphics2D g2d, boolean sign, String[] text, int width, int height) {
        // 初始化字体
        Font font = new Font("Arial", Font.BOLD, 1);
        g2d.setFont(font);
        FontMetrics metrics = g2d.getFontMetrics();
        // 字体最大、最小限制
        int minFontSize = 1;
        int maxFontSize = 100;
        // 动态调整字体大小以适应文本区域
        for (int fontSize = minFontSize; fontSize <= maxFontSize; fontSize++) {
            font = new Font("Arial", Font.BOLD, fontSize);
            g2d.setFont(font);
            metrics = g2d.getFontMetrics();
            pictureFile.setTextSize(fontSize);
            // 确保绘制区域能够容纳文本，考虑字体长度和高度
            if (sign) {
                // 高度考虑上下边距
                if (metrics.stringWidth(text[0]) > width || (metrics.getHeight() - 4) > height) {
                    // 设置字体大小
                    font = new Font("Arial", Font.BOLD, --fontSize);
                    g2d.setFont(font);
                    metrics = g2d.getFontMetrics();
                    pictureFile.setTextSize(fontSize);
                    return metrics;
                }
            } else {
                // 高度只考虑上边距
                if (metrics.stringWidth(text[0]) > width || (metrics.getHeight() - 1) * text.length > height) {
                    // 设置字体大小
                    font = new Font("Arial", Font.BOLD, --fontSize);
                    g2d.setFont(font);
                    metrics = g2d.getFontMetrics();
                    pictureFile.setTextSize(fontSize);
                    return metrics;
                }
            }
        }
        // 若字体大小超出限制，则返回最小字体
        return metrics;
    }

    /**
     * 根据图片、文字内容、图片分辨率、绘制方向、区域坐标生成带有文字的图片
     *
     * @param image        图片
     * @param text         内容数组
     * @param sign         绘制方向
     * @param topLeftX     区域左上角x坐标
     * @param topLeftY     区域左上角y坐标
     * @param bottomRightX 区域右下角x坐标
     * @param bottomRightY 区域右下角y坐标
     */
    public static void createImageWithText(InduceTemplate pictureFile, BufferedImage image, String[] text, boolean sign,boolean updown, int topLeftX, int topLeftY, int bottomRightX, int bottomRightY) {
        // 创建Graphics2D对象
        Graphics2D g2d = image.createGraphics();
        // 转换坐标，原始坐标从底部开始，转换为顶部坐标
        topLeftY = image.getHeight() - topLeftY;
        bottomRightY = image.getHeight() - bottomRightY;
        System.out.println("转换后坐标：" + topLeftY + " " + bottomRightY);
        // 计算绘制区域总宽度和高度
        int width = bottomRightX - topLeftX;
        int height = bottomRightY - topLeftY;
        // 动态调整字体大小，以适应文本区域
        FontMetrics metrics = autoFont(pictureFile, g2d, sign, text, width, height);
        // 绘制坐标点
        int textWidth;
        // 考虑文字绘制方向、文字居中。垂直居中要根据文字高度，水平居中要根据文字宽度。
//        int currentX = topLeftX + (width - textWidth) / 2;
        int currentX;
        int currentY = topLeftY + metrics.getAscent();
        // 设置文字颜色
        g2d.setColor(Color.green);
        // 根据方向开始文字绘制
        if (sign) {
            // 因文字动态调整的字体大小，所以此处代码不应该被执行
//            if (currentX < topLeftX) {
//                currentX = topLeftX; // 若超出左边界，调整为左边界
//            } else if (currentX + textWidth > topLeftX + width) {
//                currentX = topLeftX + width - textWidth; // 若超出右边界，调整为右边界
//            }
            textWidth = metrics.stringWidth(text[0]);
            currentX = topLeftX + (width - textWidth) / 2;
            // 计算居中的y坐标
            currentY = currentY + (height - metrics.getHeight()) / 2;
            if (updown) {
                currentX -= 33;
            }
            g2d.drawString(text[0], currentX, currentY);
        } else {
            // 计算每个文本内容的竖向绘制坐标
            for (String con : text) {
                // 考虑每个字符的宽度，计算居中X坐标
                textWidth = metrics.stringWidth(con);
                currentX = topLeftX + (width - textWidth) / 2;
                if (con.contains("km/h")) {
                    currentX = currentX + metrics.getDescent();
                }
                g2d.drawString(con, currentX, currentY);
                // 每绘制一个字符后Y坐标向下移，间距为文字的0.8倍
                currentY += (int) (metrics.getHeight() * 0.8);
            }
        }
        // 释放资源
        g2d.dispose();
    }

    /**
     * 在指定位置添加文字到图像
     *
     * @param image  图像对象
     * @param text   要添加的文字
     * @param x      文字的x坐标
     * @param y      文字的y坐标
     * @param size   字号
     * @param isSign 横纵向
     */
    public static void addTextToImage(BufferedImage image, String[] text, int x, int y, int size, boolean isSign) {
        // 创建Graphics对象
        Graphics g = image.getGraphics();
        // 设置字体和颜色
        g.setFont(new Font("Arial", Font.BOLD, size));
        g.setColor(Color.green);
        // 计算字体的高度，用于调整y坐标
        FontMetrics metrics = g.getFontMetrics(g.getFont());
        int textHeight = metrics.getHeight();
        // 绘制位考虑文字的高度，防止文字被裁剪
        y = (image.getHeight() - y) + (textHeight - metrics.getDescent());
        if (isSign) { // 横向
            // 在指定位置绘制文
            g.drawString(text[0], x, y);
        } else { // 纵向
            // 字体高度的70%
            int verticalSpacing = (int) (textHeight * 0.95);
            // 逐个字符绘制文字
            if(text[0].split("-").length==2) {
                g.drawString(" "+text[0].split("-")[0], x, y);
                g.drawString("  |", x, y + (1 * verticalSpacing));
                g.drawString(" "+text[0].split("-")[1].replace("km/h",""), x, y + (2 * verticalSpacing));
                g.drawString("km/h", x, y + (3 * verticalSpacing));
            }else{
                g.drawString(text[0], x, y);
            }
//            for (int i = 0; i < text[0].split("-")[1]; i++) {
//                if (i%2 == 0) {
//                    g.drawString(text[i], x, y);
//                } else {
//                    g.drawString(text[i], x, y + (i * verticalSpacing));
//                }
//            }
        }
        // 释放资源
        g.dispose();
    }

    /**
     * 将JPG文件转换为BMP格式
     *
     * @param jpgFilePath 输入的JPG文件路径
     * @param bmpFilePath 输出的BMP文件路径
     * @throws IOException 转换失败时抛出异常
     */
    public static void convertJpgToBmp(String jpgFilePath, String bmpFilePath) throws IOException {
        // 读取JPG文件为BufferedImage
        BufferedImage jpgImage = ImageIO.read(new File(jpgFilePath));
        if (jpgImage == null) {
            throw new IOException("无法读取JPG文件: " + jpgFilePath);
        }
        // 写入BMP文件
        File bmpFile = new File(bmpFilePath);
        if (ImageIO.write(jpgImage, "bmp", bmpFile)) {
            System.out.println("JPG已成功转换为BMP: " + bmpFilePath);
        } else {
            throw new IOException("无法将JPG写入BMP文件！");
        }
    }

    /**
     * 组装mqtt消息
     *
     * @param dataModel 传输参数
     * @param templateName 模板名称
     * @return
     */
    private String convertMessage(String templateName,Object dataModel) throws IOException, TemplateException {
        Template template = freemarkerConfig.getTemplate(templateName);
        StringWriter writer = new StringWriter();
        template.process(dataModel, writer);
        return writer.toString();
    }

    private String convertMessage(MessageParam param) {
        int flg = param.getFlg();
        String sourceid = param.getSourceId();
        String vmsid = param.getEquipCode();
        String type = param.getType();
        if (flg == 1) {
            Integer x = param.getX();
            Integer y = param.getY();
            Integer width = param.getResolutionWidth();
            Integer height = param.getResolutionHeight();
            Integer playorder = param.getPlayorder();
            Integer duration = param.getDuration();
            String ftpPath = param.getFtpPath();
            String upMessage = "<?xml version=\"1.0\" encoding=\"GB2312\"?>\n" +
                    "<message System=\"ATMS\" Ver=\"1.0\">\n" +
                    "\t<systemtype>VMS</systemtype>\n" +
                    "\t<messagetype>CONTROL</messagetype>\n" +
                    "\t<sourceIP>"+sourceid+"</sourceIP>\n" +
                    "\t<targetIP></targetIP>\n" +
                    "\t<user></user>\n" +
                    "\t<password></password>\n" +
                    "\t<task>\n" +
                    "\t\t<tasktype>updatelistandfileEx</tasktype>\n" +
                    "\t\t\t<taskdetail>\n" +
                    "\t\t\t\t<ledid width=\""+width+"\" height=\""+height+"\">"+vmsid+"</ledid>\n" +
                    "\t\t\t\t<list type=\"composite\">\n" +
                    "\t\t\t\t\t<file playorder=\""+playorder+"\" duration=\""+duration+"\" starttime=\"2024-11-07 00:00:00\" endtime=\"2030-11-07 23:59:59\" backgroundcolor=\"#000000\" url=\"\">\n" +
                    "\t\t\t\t\t\t<area contenttype=\"imageFile\" x=\""+x+"\" y=\""+y+"\" w=\""+width+"\" h=\""+height+"\" entermode=\"0\" enterspeed=\"1\" exitmode=\"0\" exitspeed=\"0\">\n" +
                    "\t\t\t\t\t\t\t<imagefile filetype=\"bmp\" type=\"uri\">"+ftpPath+"</imagefile>\n" +
                    "\t\t\t\t\t\t</area>\n" +
                    "\t\t\t\t\t</file>\n" +
                    "\t\t\t\t</list>\n" +
                    "\t\t\t</taskdetail>\n" +
                    "\t\t<senderinfo>"+type+"</senderinfo>\n" +
                    "\t</task>\n" +
                    "</message>";

            return upMessage;
        } else {
            String downMessage = "<?xml version=\"1.0\" encoding=\"gb2312\"?>\n" +
                    "<message System=\"ATMS\" Ver=\"1.0\">\n" +
                    "<systemtype>VMS</systemtype>\n" +
                    "<messagetype>CONTROL</messagetype>\n" +
                    "<!--token-->\n" +
                    "<sourceIP>"+sourceid+"</sourceIP>\n" +
                    "<targetIP></targetIP>\n" +
                    "<user></user>\n" +
                    "<password></password>\n" +
                    "<task>\n" +
                    "<tasktype>deletelistandfile</tasktype>\n" +
                    "<taskdetail>\n" +
                    "<!--屏体ID 由现场提供-->\n" +
                    "      <ledid>"+vmsid+"</ledid>  \n" +
                    "</taskdetail>\n" +
                    "<!--节目类型 TFM 路况 TFMH 独占 TAM 交通事件-->\n" +
                    "<senderinfo>"+type+"</senderinfo>\n" +
                    "</task>\n" +
                    "</message>";
            return downMessage;
        }

    }


}
